@@ -1,4 +1,7 @@
-from xml.etree import ElementTree
+import datetime
+import json
+import os
+import hashlib
 
 import requests
 
@@ -6,9 +9,19 @@ from quickbase_json.helpers import FileUpload, Where, QBFile, split_list_into_ch
 from quickbase_json.qb_insert_update_response import QBInsertResponse
 from quickbase_json.qb_response import QBQueryResponse
 
+QUERY_CACHE = 'query_cache'
+
+try:
+    import pkg_resources
+
+    version = pkg_resources.require("quickbase-json-api-client")[0].version
+except Exception as e:
+    print(e)
+    version = 'unknown'
+
 
 class QuickbaseJSONClient:
-    def __init__(self, realm, auth, agent: str = 'QJAC', debug=False, **kwargs):
+    def __init__(self, realm, auth, agent: str = f'python-qjac/{version}', debug=False, **kwargs):
         """
         Creates a client object.
         :param realm: quickbase realm
@@ -76,6 +89,49 @@ class QuickbaseJSONClient:
 
         return res
 
+    def cache_query(self, table: str, select: list, where: any, hours: float, **kwargs):
+        """
+        Caches a query for a given amount of time.
+        https://developer.quickbase.com/operation/runQuery
+        :param table: quickbase table
+        :param select: list, list of fids to query
+        :param where: Quickbase query language string. i.e. {3.EX.100}
+        :param hours: Number of hours to cache the query for.
+        :param kwargs: optional request parameters.
+        :return: json data of records {data: ..., fields: ...}
+        """
+
+        # create query_cache directory if it does not exist
+        if not os.path.exists(QUERY_CACHE):
+            os.makedirs(QUERY_CACHE)
+
+        # get query hash
+        h = hashlib.md5(f'{table}_{select}_{where}'.encode('utf-8')).hexdigest()
+
+        # get last updated, hours
+        query_file = f'{QUERY_CACHE}/{h}.json'
+        if os.path.exists(query_file):
+            query_json = json.load(open(query_file, 'r')) if os.path.exists(query_file) else {}
+            last_updated = datetime.datetime.fromtimestamp(query_json.get('last_updated'))
+            cache_hours = query_json.get('hours', None)
+            # check if query is cached and if it is not expired
+            if not datetime.datetime.now() - last_updated > datetime.timedelta(hours=cache_hours):
+                res = QBQueryResponse()
+                res.update({'data': query_json['response']})
+                res.ok = True
+                return res
+
+        else:
+            # query is not cached or expired, run query and cache it
+            res = self.query_records(table, select, where, **kwargs)
+            data = {
+                'last_updated': datetime.datetime.now().timestamp(),
+                'hours': hours,
+                'response': res.data()
+            }
+            open(f'{QUERY_CACHE}/{h}.json', 'w').write(json.dumps(data))
+            return res
+
     def insert_update_records(self, table: str, data: list):
         """
         Inerts or updates records in a given table.
@@ -103,10 +159,11 @@ class QuickbaseJSONClient:
         :param table: The unique identifier of the table.
         :param where: The filter to delete records. To delete all records specify a filter that will include all records.
         :return: dict, numberDeleted is number of records deleted.
+        :
         """
 
         headers = self.headers
-        body = {'to': table, 'where': where}
+        body = {'from': table, 'where': where}
 
         if self.debug:
             print(f'QJAC : delete_records : body ---> \n{body}')
