@@ -25,30 +25,47 @@ def operation(method):
 
 class QBResponse(dict):
     def __init__(self, requests_response=None, **kwargs):
-        super().__init__()
+        self.ok = False
         if requests_response:
             self.ok = requests_response.ok
             self.status_code = requests_response.status_code
             self.text = requests_response.text
+        super().__init__()
 
 
 class QBQueryResponse(QBResponse):
 
     def __init__(self, res=None, **kwargs):
-        self.response_type = 'records'
-        self.operations = []
         # potential to load sample data for testing
         if kwargs.get('sample_data'):
             self.update(kwargs.get('sample_data'))
+            self.ok = True
+            self.status_code = 200
+
+        # load response data
+        if res is not None:
+            self.ok = res.ok
+            self.status_code = res.status_code
+            self.text = res.text
+
+        self.response_type = 'records'
+        self.operations = []
         super().__init__(requests_response=res)
 
     def is_empty(self):
         """
-        Tests if data is empty, return True if so.
+        Tests if data is empty or otherwise can't be accessed.
         :return: boolean
         """
         return self.get('data', False)
 
+    def first(self) -> dict:
+        """
+        Gets the first record in the data.
+        :return: dict
+        """
+        return self.data()[0]
+
     def info(self, prt=True):
         """
         Prints information about the response.
@@ -88,7 +105,7 @@ class QBQueryResponse(QBResponse):
 
             return info_str
 
-    def fields(self, prop):
+    def fields(self, prop) -> list:
         """
         Gets fields, given property, i.e. label, id or type
         :param prop: property of field
@@ -96,14 +113,21 @@ class QBQueryResponse(QBResponse):
         """
         return [i.get(prop) for i in self.get('fields')]
 
-    def data(self):
+    def data(self, key_type='str') -> list:
         """
         Gets data, shorthand for .get('data')
         :return: data
         """
-        return self.get('data')
+        data = self.get('data', False)
+        if key_type == 'int':
+            return [{int(k): v for k, v in d.items()} for d in data]
+        elif key_type == 'str' or key_type == 'string':
+            return [{str(k): v for k, v in d.items()} for d in data]
+        else:
+            raise ValueError('Invalid key type.')
 
-    def prd(self, data):
+    @staticmethod
+    def prd(data):
         """
         Print relevant data, simple print function.
         :param data: data to print
@@ -111,7 +135,7 @@ class QBQueryResponse(QBResponse):
         print("\033[91m", 'Relevant Data:\n\n', data, '\n', "\033[0m")
 
     @operation
-    def denest(self):
+    def denest(self) -> QBResponse:
         """
         Denests data, i.e. if in {'key': {'value': actual_value}} format. -> {'key': actual_value}
         :return: QBResponse
